#!/usr/bin/env bash

#
# Copyright (c) 2009 Peter Ljung <ljung.peter@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

usage()
{
cat << EOF
usage: $0 options

This script output file status for SwingCVS application. Single file(s) update is not supported.

OPTIONS:
   -h   show help
   -p   root path used by finder
   -l   show status only for local folder (default is recursively)
   -v   verbose/debug output
EOF
}

while getopts "hp:lv" OPTION; do
  case $OPTION in
    h )  usage;exit 1;;
    p )  ROOT=$OPTARG;;
    l )  LOCAL=1;;
    v )  VERBOSE=1;;
    ? )  usage;exit 1;;
  esac
done

# Helper: Execute command
# Executor executes command (including paramenters) sent to e
# Parameters including space are escaped with ""
function e {
	CMD=""
	for p in "$@";
  do
	  p="${p/# /}"									# Strip beginning/trailing spaces
	  p="${p/% /}"
	  if [[ $p =~ " " ]]; then
		  CMD="$CMD \"$p\"";
		else
      CMD="$CMD $p";
		fi
  done
  if [[ $VERBOSE ]]; then
  	echo "eval: $CMD"
	fi
  eval $CMD
}

# Use pwd as root if not specified explicitely
if [ -z "${ROOT}" ]; then
  ROOT="$PWD"
fi
ROOT="${ROOT/# /}"								# Strip beginning/trailing space
ROOT="${ROOT/% /}"
ROOT="${ROOT/%\//}"               # Remove trailing / (% means comparison from end)

e cd "$ROOT"

if [ ${LOCAL} ]; then
  CMD="cvs status -l 2>&1"
else
  CMD="cvs status -R 2>&1"
fi

CMD=$CMD" | egrep 'Examining|Status|Working'"
CMD=$CMD" | sed 's/^.*Examining[[:space:]]*/D;/'"        			# Replace beginning of "Examining" lines with D;
CMD=$CMD" | sed 's/^.*File:[[:space:]]*/F;/'"            			# Replace "File:" with F;
CMD=$CMD" | sed 's/[[:space:]]*Status:[[:space:]]*/;/'"  			# Replace "Status:" with ;
CMD=$CMD" | sed 's/^.*Working.*:[[:space:]]*/;/'" 			 			# Replace beginning of "Working" lines with ;
# TODO: Add support for repository revision

if [ ${VERBOSE} ]; then
  echo "Executing: $CMD"
fi

echo "R;$ROOT"
# Evaluate cmd and concatenate F; and W; lines (failed to include awk command in CMD string)
eval $CMD | awk '/^F;/{L=$0};/^;/{print L$0};/^R;/;/^D;/'
