#!/bin/bash
# #!/usr/local/bin/bash

# Not /bin/ksh OR /bin/sh compatible. bash is not default in OpenBSD

usage()
{
cat << EOF
usage: $0 options folder

This script locates files for SwingCVS application.

OPTIONS:
   -h   show help
   -p   root path used by finder
#   -r   recursive (always?)
   -i   include filter e.g. "*.c,*.h"
   -e   exclude filter e.g. "*.ini,*~" (precedence over include)
   -v   verbose/debug output
EOF
}

while getopts "hp:ri:e:v" OPTION; do
  case $OPTION in
    h )  usage;exit 1;;
    p )  ROOT=$OPTARG;;
#    r )  ;;
    i )  INCLUDE=$OPTARG;;
    e )  EXCLUDE=$OPTARG;;
    v )  VERBOSE=1;;
    ? )  usage;exit 1;;
  esac
done

# Use pwd as root if not specified explicitely
if [ -z ${ROOT} ]; then
  ROOT="$PWD"
fi
ROOT="${ROOT/%\//}"               # remove trailing / (% means comparison from end)

# If no folder/file has been entered input is assumed from STDIN
if [ ${!OPTIND} ]; then
  FOLDER=${!OPTIND}
  FOLDER="${FOLDER/%\//}"         # remove trailing / (% means comparison from end)
fi

# Reformat "loose" regexp format to common UNIX egrep regexp format
if [ ${INCLUDE} ]; then
  INCLUDE="${INCLUDE//,/|}"       # replace , with |
  INCLUDE="${INCLUDE//\./\\.}"    # replace . with \.
  INCLUDE="${INCLUDE//\*/.*}"     # replace * with .*
fi
if [ ${EXCLUDE} ]; then
  EXCLUDE="${EXCLUDE//,/|}"
  EXCLUDE="${EXCLUDE//\./\\.}"
  EXCLUDE="${EXCLUDE//\*/.*}"
fi

if [ ${FOLDER} ]; then

  FOLDER="$ROOT/$FOLDER"
  # Output root folder while removing /folder/.. and /.
  echo "R;$FOLDER" | sed 's/\/[^\/]*\/\.\.//' | sed 's/\/\.//'

  CMD="ls -lR $FOLDER"
  CMD=$CMD" | grep -v ^[[:space:]]\*$"                 # Remove blank lines
  CMD=$CMD" | grep -v ^total"                          # Remove total lines
  CMD=$CMD" | grep -v ^d"                              # Remove directory files
  CMD=$CMD" | sed '/:$/s/^/D;/g'"                      # Add D; to folder lines
  CMD=$CMD" | sed 's/:$//g'"			       	       # Remove trailing :
  CMD=$CMD" | sed 's/^.*[0-9][0-9]:[0-9][0-9] /F;/g'"  # Replace beginning of file lines with F;
  if [ ${EXCLUDE} ]; then
    CMD=$CMD" | egrep -v \"^F;.*$EXCLUDE\""	           # Exclude filter (only files can be excluded)
  fi
  if [ ${INCLUDE} ]; then
    CMD=$CMD" | egrep \"^R;|^D;|^F;.*$INCLUDE\""       # Include filter (root and directories are kept)
  fi
  CMD=$CMD" | sed 's/\/[^\/]*\/\.\.//'"                # Remove /folder/..
  CMD=$CMD" | sed 's/\/\.//'"                          # Remove /.

  if [ ${VERBOSE} ]; then
    echo "ROOT=$ROOT"
    echo "FOLDER=$FOLDER"
    echo "INCLUDE=$INCLUDE"
    echo "EXCLUDE=$EXCLUDE"
    echo "Execute: $CMD"
    echo ""
  fi

  eval $CMD

else

  echo "No file/folder was entered. File name input from STDIN not supported yet. Exit"
  exit 1

fi

