#!/bin/bash
# #!/usr/local/bin/bash

# Not /bin/ksh OR /bin/sh compatible. bash is not default in OpenBSD

# TODO: This solution could probably simplified by e.g. awk

usage()
{
cat << EOF
usage: $0 options folder

This script takes the output (via STDIN) defined by scvs-finder.sh and reverse the order of items to make it suitable for cvs remove.

Reverse order means that;
* Folders lower in the hierarchy are output first
* R; and D; lines are still before the files

OPTIONS:
   -h   show help
   -v   verbose/debug output
EOF
}

while getopts "hv" OPTION; do
  case $OPTION in
    h )  usage;exit 1;;
    v )  VERBOSE=1;;
    ? )  usage;exit 1;;
  esac
done

# Create temporary file and store input from STDIN to enable it to be read several times
TMPFILE=`mktemp -t scvs1.XXXXXXXXXX` || exit 1
TMPFILE2=`mktemp -t scvs2.XXXXXXXXXX` || exit 1
cat <&0 > $TMPFILE

# Print first R; and F; lines and remove the same section from $TMPFILE
echo "" > $TMPFILE2
cat $TMPFILE | awk "BEGIN {S=0};/^D;/{S=1};{if(S==0)print}"
cat $TMPFILE | awk "BEGIN {S=0};/^D;/{S=1};{if(S!=0)print}" >> $TMPFILE2
cat $TMPFILE2 > $TMPFILE

DIR="NONE"
while [[ $DIR != "D;" ]]; do

  # Find folder line which has no decendants
  DIR="D;"
  while read LINE; do
    if [[ $LINE =~ ^$DIR* ]]; then
      DIR=$LINE
    fi
  done < $TMPFILE

  # Print section between $DIR and "D;" after $DIR and remove the same section from $TMPFILE
  echo "" > $TMPFILE2
  cat $TMPFILE | awk "BEGIN {S=0};/^D;/{if(S==1)S=2};/^${DIR//\//\/}/{S=1};{if(S==1)print}" 
  cat $TMPFILE | awk "BEGIN {S=0};/^D;/{if(S==1)S=2};/^${DIR//\//\/}/{S=1};{if(S!=1)print}" >> $TMPFILE2 
  cat $TMPFILE2 > $TMPFILE

done

# Remove temporary file
rm -f $TMPFILE	
rm -f $TMPFILE2	

