#!/usr/local/bin/bash

# Not /bin/ksh OR /bin/sh compatible. bash is not default in OpenBSD

usage()
{
cat << EOF
usage: $0 options

This script output file status for SwingCVS application. Single file(s) update is not supported.

OPTIONS:
   -h   show help
   -p   root path used by finder
   -l	show status only for local folder (default is recursively)
   -v   verbose/debug output
EOF
}

while getopts "hp:lv" OPTION; do
  case $OPTION in
    h )  usage;exit 1;;
    p )  ROOT=$OPTARG;;
    l )  LOCAL=1;;
    v )  VERBOSE=1;;
    ? )  usage;exit 1;;
  esac
done

# Use pwd as root if not specified explicitely
if [ -z ${ROOT} ]; then
  ROOT="$PWD"
fi
ROOT="${ROOT/%\//}"               # remove trailing / (% means comparison from end)

CMD="cd $ROOT"
if [ ${VERBOSE} ]; then
  echo "Executing: $CMD"
fi

eval $CMD

if [ ${LOCAL} ]; then
  CMD="ls -Tl"
else
#  CMD="ls -TlR"
  echo "-R option is not supported"
fi
CMD=$CMD" | grep -v ^d"
CMD=$CMD" | awk '{print \$10,\$6,\$7,\$8,\$9}'"
CMD=$CMD" | sed 's/ /;/'"			       # Replace ' ' with ;
CMD=$CMD" | sed 's/[0-9][0-9]:/xx:/'"                  # Replace HH: with xx: (CVS use UTC while ls use local timezone)

if [ ${VERBOSE} ]; then
  echo "Executing: $CMD"
fi

echo "R;$ROOT"
eval $CMD

CMD="cat ./CVS/Entries"
CMD=$CMD" | grep '//'"                                 # Only keep file lines
CMD=$CMD" | sed 's/^\///'"                             # Remove first /
CMD=$CMD" | sed 's/\/[^\/]*\/[^ ]*//'"                 # Replace first '/x.y/Day '
CMD=$CMD" | sed 's/\/\/.*$//'"                         # Remove trailing //
CMD=$CMD" | sed 's/ /;/'"                              # Replace ' ' with ;
CMD=$CMD" | sed 's/[0-9][0-9]:/xx:/'"                  # Replace HH: with xx: (CVS use UTC while ls use local timezone) 

if [ ${VERBOSE} ]; then
  echo "Executing: $CMD"
fi
eval $CMD

# It doesn't work to sort within the script do | sort | uniq -d outside the script
# For some reason there is a time difference of 2 hours even on up-to-date files

#CMD="sort <1"

#if [ ${VERBOSE} ]; then
#  echo "Executing: $CMD"
#fi
#eval $CMD

