#!/bin/bash

# #!/usr/local/bin/bash
# Not /bin/ksh OR /bin/sh compatible. bash is not default in OpenBSD

usage()
{
cat << EOF
usage: $0 options

This script output file status for SwingCVS application. Single file(s) update is not supported.

OPTIONS:
   -h   show help
   -p   root path used by finder
   -l	show status only for local folder (default is recursively)
   -v   verbose/debug output
EOF
}

while getopts "hp:lv" OPTION; do
  case $OPTION in
    h )  usage;exit 1;;
    p )  ROOT=$OPTARG;;
    l )  LOCAL=1;;
    v )  VERBOSE=1;;
    ? )  usage;exit 1;;
  esac
done

# Use pwd as root if not specified explicitely
if [ -z ${ROOT} ]; then
  ROOT="$PWD"
fi
ROOT="${ROOT/%\//}"               # remove trailing / (% means comparison from end)

CMD="cd $ROOT"
if [ ${VERBOSE} ]; then
  echo "Executing: $CMD"
fi

eval $CMD

if [ ${LOCAL} ]; then
  CMD="cvs status -l 2>&1"
else
  CMD="cvs status -R 2>&1"
fi

CMD=$CMD" | egrep 'Examining|Status'"
CMD=$CMD" | sed 's/^.*Examining[[:space:]]*/D;/'"        # Replace beginning of "Examining" lines with D;
CMD=$CMD" | sed 's/^.*File:[[:space:]]*/F;/'"            # Replace "File:" with F;
CMD=$CMD" | sed 's/[[:space:]]*Status:[[:space:]]*/;/'"  # Replace "Status:" with ;

if [ ${VERBOSE} ]; then
  echo "Executing: $CMD"
fi

echo "R;$ROOT"
eval $CMD

